@@ -10,6 +10,7 @@
 #define EXTUNIX_WANT_PWRITE
 #define EXTUNIX_WANT_READ
 #define EXTUNIX_WANT_WRITE
+#define EXTUNIX_WANT_GETTID
 #include "config.h"
 
 #if defined(EXTUNIX_HAVE_TTYNAME)
@@ -31,7 +32,7 @@ CAMLprim value caml_extunix_ttyname(value v_fd)
 
 #if defined(EXTUNIX_HAVE_CTERMID)
 
-CAMLprim value caml_extunix_ctermid(value v_unit) 
+CAMLprim value caml_extunix_ctermid(value v_unit)
 {
   char buf[L_ctermid + 1];
   UNUSED(v_unit);
@@ -40,6 +41,16 @@ CAMLprim value caml_extunix_ctermid(value v_unit)
 
 #endif
 
+#if defined(EXTUNIX_HAVE_GETTID)
+
+CAMLprim value caml_extunix_gettid(value v_unit)
+{
+  UNUSED(v_unit);
+  return Val_int(syscall(SYS_gettid));
+}
+
+#endif
+
 #if defined(EXTUNIX_HAVE_PGID)
 
 CAMLprim value caml_extunix_setpgid(value v_pid, value v_pgid)
